@@ -29,10 +29,10 @@ __PACKAGE__->validates(first_name => (presence=>1, length=>[2,24]));
 __PACKAGE__->validates(last_name => (presence=>1, length=>[2,48]));
 
 __PACKAGE__->validates(credit_cards => (result_set=>+{validations=>1, min=>2, max=>4}, on=>'profile' ));
-__PACKAGE__->accept_nested_for('credit_cards');
+__PACKAGE__->accept_nested_for('credit_cards', +{allow_destroy=>1});
 
 __PACKAGE__->validates(person_roles => (result_set=>+{validations=>1, min=>1}, on=>'profile' ));
-__PACKAGE__->accept_nested_for('person_roles');
+__PACKAGE__->accept_nested_for('person_roles', {allow_destroy=>1});
 
 __PACKAGE__->validates(profile => (result=>+{validations=>1}, allow_blank=>1 ));
 __PACKAGE__->accept_nested_for('profile');
@@ -60,7 +60,7 @@ __PACKAGE__->has_many(
 
 __PACKAGE__->many_to_many('roles' => 'person_roles', 'role');
 
-__PACKAGE__->accept_nested_for('roles');
+__PACKAGE__->accept_nested_for('roles', {allow_destroy=>1});
 
 # TODO: I think these can be removed
 sub registered {
