@@ -11,6 +11,8 @@ use Scalar::Util 'blessed';
 use Carp;
 use namespace::autoclean -also => ['debug'];
 use DBIx::Class::Valiant::Util::Exception::TooManyRows;
+use DBIx::Class::Valiant::Util::Exception::BadParameterFK;
+use DBIx::Class::Valiant::Util::Exception::BadParameters;
 
 with 'DBIx::Class::Valiant::Validates';
 with 'Valiant::Filterable';
@@ -123,6 +125,8 @@ sub insert {
   if($self->errors->size) {
     debug 2, "Skipping insert for @{[$self]} because its invalid";
     return $self;
+  } else {
+    debug 2, "proceeding to insert  @{[$self]} because its valid";
   }
 
   if($self->{__valiant_donot_insert}) {
@@ -132,6 +136,33 @@ sub insert {
   return $self->next::method(@args);
 }
 
+sub _nested_info_for_related {
+  my ($self, $related) = @_;
+  my %nested = $self->result_class->accept_nested_for;
+  my %info = %{ $nested{$related}||+{} };  
+  return %info;
+}
+
+sub _related_allow_destroy {
+  my ($self, $related) = @_;
+  my %info = $self->_nested_info_for_related($related);
+  if(my $proto = $info{allow_destroy}) {
+    my $allow_or_not = (ref($proto)||'' eq 'CODE') ? $proto->($self) : $proto;
+    return $allow_or_not;
+  }
+  return 0;
+}
+
+sub _related_limit {
+  my ($self, $related) = @_;
+  my %info = $self->_nested_info_for_related($related);
+  if(my $limit_proto = $info{limit}) {
+    my $limit = (ref($limit_proto)||'' eq 'CODE') ? $limit_proto->($self) : $limit_proto;
+    return 1, $limit;
+  }
+  return 0, undef;
+}
+
 sub update {
   my ($self, $upd) = @_;
   my $context = delete($upd->{__context})||[];
@@ -167,12 +198,10 @@ sub update {
     if(my $cb = $nested{$related}->{reject_if}) {
       next if $cb->($self, $related{$related});
     }
-    if(my $limit_proto = $nested{$related}->{limit}) {
-      my $limit = (ref($limit_proto)||'' eq 'CODE') ?
-        $limit_proto->($self) :
-        $limit_proto;
-      my $num = scalar @{$related{$related}};
 
+    my ($has_limit, $limit) = $self->_related_limit($related);
+    if($has_limit) {
+      my $num = scalar @{$related{$related}};
       DBIx::Class::Valiant::Util::Exception::TooManyRows
         ->throw(
           limit=>$limit,
@@ -180,8 +209,8 @@ sub update {
           related=>$related,
           me=>$self->result_source->name,
         ) if $num > $limit;
-     
     }
+
     debug 2, "Setting related '$related' for @{[ ref $self ]} ";
 
     #$self->update_or_create_related($related, $related{$related});
@@ -194,7 +223,11 @@ sub update {
   debug 2, "About to run validations for @{[$self]} on update";
   $self->validate(%validate_args) if $self->auto_validation;
 
-  return $self if $self->errors->size;
+
+  if($self->has_errors) {
+    debug 2, "Validation errors found, skipping mutate for @{[ ref $self ]} ";
+    return $self;
+  }
   debug 2, "No validation issues found, proceeding to mutate @{[ ref $self ]} ";
 
   # wrap it all in a transaction to undo if issues.
@@ -313,6 +346,7 @@ sub is_unique {
 
 sub mark_for_deletion {
   my ($self) = @_;
+  return unless $self->{__valiant_allow_destroy};
   $self->{__valiant_kiss_of_death} = 1;
 }
 
@@ -529,13 +563,19 @@ sub set_multi_related_from_params {
     die "We expect '$params' to be some sort of reference but its not!";
   }
 
+  my $allow_destroy = $self->_related_allow_destroy($related);
+
   # Queue up some meta data here just once.  We get existing rows ans
   # uniqiue key info (including PK info)
   debug 2, "looking for $related cached or existing rows";
   my @existing_rows = @{ $self->$related->get_cache||[] };
   unless(@existing_rows) {
-    debug 2, "cache was empty so going to check DB"; ## TODO this is to support ->discard_changes but maybe not needed
-    @existing_rows = $self->$related->all;
+    debug 3, "cache was empty so going to check DB"; ## TODO this is to support ->discard_changes but maybe not need
+    if($self->in_storage) {
+      @existing_rows = $self->$related->all;
+    } else {
+      debug 3, "record not in storage, so no point in looking for related rows";
+    }
   }
   debug 2, "Found @{[ scalar @existing_rows ]} existing rows for $related";
   
@@ -566,6 +606,48 @@ sub set_multi_related_from_params {
       debug 3, "Is a NOP row, so skipping";
       next;
     }
+
+    # Ok so if $self is in storage we expect any $related FKs to match whatever is $self $PK.  If that
+    # FK is empty we add it.   If its there but has an unexpected value, that's an error (and probably
+    # a hacking attempt
+    if($self->in_storage) {
+      my %cond = %{$self->result_source->relationship_info($related)->{cond}};
+      foreach my $fk_proto (keys %cond) {
+        my $pk_proto = $cond{$fk_proto};
+        my ($pk) = ($pk_proto=~m/^self\.(.+)$/); 
+        my ($fk) = ($fk_proto=~m/^foreign\.(.+)$/);
+
+        if(exists $param_row->{$fk}) {
+          unless($param_row->{$fk} eq $self->get_column($pk)) {
+            DBIx::Class::Valiant::Util::Exception::BadParameterFK->throw(
+              fk_field=>$fk, fk_value=>$param_row->{$fk}, 
+              pk_field=>$pk, pk_value=>$self->get_column($pk), 
+              related=>$related, me=>$self);
+          }
+        } else {
+          $param_row->{$fk} = $self->get_column($pk); # set it since that should help with the lookups
+        }
+      }
+    } else {
+      # Now if $self is not in storage we expect the related FK to be empty.  If not throw error since
+      # its possible a hacking attempt
+      my %cond = %{$self->result_source->relationship_info($related)->{cond}};
+      foreach my $fk_proto (keys %cond) {
+        my $pk_proto = $cond{$fk_proto};
+        my ($pk) = ($pk_proto=~m/^self\.(.+)$/); 
+        my ($fk) = ($fk_proto=~m/^foreign\.(.+)$/);
+
+        if(exists $param_row->{$fk}) {
+          unless(!defined($param_row->{$fk})) {
+            DBIx::Class::Valiant::Util::Exception::BadParameterFK->throw(
+              fk_field=>$fk, fk_value=>$param_row->{$fk}, 
+              pk_field=>$pk, pk_value=>'undef', 
+              related=>$related, me=>$self);
+          }
+        }
+      }
+    }
+
     my $was_add = delete $param_row->{_add};
     my $related_model;
     if(blessed $param_row) {
@@ -603,9 +685,10 @@ sub set_multi_related_from_params {
 
         # If it doesn't match in the cache then we have to find it in the DB. TODO I'm skipping this
         # lookup for now since I don't think we need it unless the user failed to properly prefetch.
+        # Its pointless to do this unless $self is in storage anyway
         unless($related_model) {
-          debug 2, "Trying to find $related in DB with key $key";
-          #$related_model = $self->find_related($related, \%matching, +{key=>$key});
+          debug 2, "Trying to find $related in DB with key $key (SKIPPING)";
+          #$related_model = $self->find_related($related, \%matching, +{key=>$key}) if $self->in_storage;
           if($related_model) {
             debug 2, "found related model for $related with $key in DB";
             %keys_used_to_find = (%keys_used_to_find, %matching);
@@ -637,6 +720,10 @@ sub set_multi_related_from_params {
         $related_model->skip_validate;
         $related_model->{__valiant_donot_insert} = 1;
       }
+
+      if($allow_destroy) {
+          $related_model->{__valiant_allow_destroy} = 1;
+      }
       
       # Don't set params for the found keys (waste of time, no change)
       my %params_for_recursive = %$param_row;
@@ -647,8 +734,16 @@ sub set_multi_related_from_params {
       # we 'might' have enough info to actually load it now.
       if(!$related_model->in_storage && !$related_model->is_marked_for_deletion) {
         # TODO: We should skip this if we've already tried a full PK
-        my $copy = $related_model->get_from_storage(); # If we now have the full PK we can find it
-        $related_model = $copy if $copy;
+        my %ident = %{ $related_model->_storage_ident_condition ||+{} };
+        my $fully_id = 1;
+        foreach my $ident(keys %ident) {
+          next if defined($related_model->get_column($ident)); # can't try to load from DB if the PK isn't fully found
+          $fully_id = 0;
+        }
+        if($fully_id) {
+          my $copy = $related_model->get_from_storage(); # If we now have the full PK we can find it
+          $related_model = $copy if $copy;
+        }
       }
     } else {
       die "Not sure what to do with $param_row";
@@ -679,13 +774,14 @@ sub set_multi_related_from_params {
     # Only mark for deletion if its actually in store.
     if($current->in_storage) {
       debug 2, "Marking $current for deletion";
+      $current->{__valiant_allow_destroy} = 1 if $allow_destroy;
       $current->mark_for_deletion;
 
       # Mark its children as pruned, recursively
       my $cb; $cb = sub {
         my $row = shift;
-        debug 3, "marking $row to be pruned";
-        $row->{__valiant_is_pruned} = 1;
+        debug 3, "marking $row to be pruned" unless $row->is_marked_for_deletion;
+        $row->{__valiant_is_pruned} = 1; # unless $row->is_marked_for_deletion;
         my @related = keys %{$row->{_relationship_data}||+{}};
         # TODO only do this for has_one, might_have, has_many
         debug 3, "$row has related data to prune @{[ join ',', @related ]}";
@@ -698,7 +794,7 @@ sub set_multi_related_from_params {
           }
         }
       };
-      $cb->($current);
+      $cb->($current) if $current->is_marked_for_deletion; # We check because 'mark_for_deletion' will skip unless 'allow_destroy' is set
     }
 
     push @related_models, $current if $current->in_storage; # don't preserve unsaved previous
@@ -714,6 +810,7 @@ sub set_multi_related_from_params {
 sub set_single_related_from_params {
   my ($self, $related, $params) = @_;
   my %nested = $self->result_class->accept_nested_for;
+  my $allow_destroy = $nested{allow_destroy};
 
   # Is there an existing related object in the cache?  If so then we
   # will merge params with existing rather than create a new one or
@@ -729,6 +826,7 @@ sub set_single_related_from_params {
     ## TODO this is probably wrong if $params has different FKs or unique fields
 
     debug 2, "Found cached related_result $related for @{[ ref $self ]} ";
+    $related_result->{__valiant_allow_destroy} = 1 if $allow_destroy;
     $related_result->set_from_params_recursively(%$params);
   } else {
     debug 2, "No cached related_result $related for @{[ ref $self ]} ";
@@ -743,8 +841,28 @@ sub set_single_related_from_params {
       # rel with a new one (find_or_new).
 
       if($self->in_storage) {
-        debug 2, "Updating related result '$related' for @{[ ref $self ]} ";
         my %local_params = %$params;
+        my %cond = %{$self->result_source->relationship_info($related)->{cond}};
+        my @pks = $self->result_source->primary_columns;
+        foreach my $fk_proto (keys %cond) {
+          my $pk_proto = $cond{$fk_proto};
+          my ($pk) = ($pk_proto=~m/^self\.(.+)$/); 
+          my ($fk) = ($fk_proto=~m/^foreign\.(.+)$/);
+          next unless grep { $_ eq $pk } @pks;;
+
+          if(exists $local_params{$fk}) {
+            unless($local_params{$fk} eq $self->get_column($pk)) {
+              DBIx::Class::Valiant::Util::Exception::BadParameterFK->throw(
+                fk_field=>$fk, fk_value=>$local_params{$fk}, 
+                pk_field=>$pk, pk_value=>$self->get_column($pk), 
+                related=>$related, me=>$self);
+            }
+          } else {
+            #$local_params{$fk} = $self->get_column($pk); # set it since that should help with the lookups
+          }
+        }
+
+        debug 2, "Updating related result '$related' for @{[ ref $self ]} ";
         my %pk = map { $_ => delete $local_params{$_} }
           grep { exists $local_params{$_} }
           $self->related_resultset($related)->result_source->primary_columns;
@@ -753,15 +871,17 @@ sub set_single_related_from_params {
         # also apply any updates to that record as indicated.
         if(%pk) {
           debug 3, "Updating with exact record matching pk";
+
           $related_result = $self->result_source->related_source($related)->resultset->find($params); ## TODO shouldnt this be \%pk??
+          $related_result = $self->result_source->related_source($related)->resultset->find(\%pk); ## TODO shouldnt this be \%pk??
+
           $self->set_from_related($related, $related_result);
 
           #my $rev_data = $self->result_source->reverse_relationship_info($related);
           #my ($reverse_related) = keys %$rev_data;
           #$related_result->set_from_related($reverse_related, $self) if $reverse_related; # Don't have this for might_have
-
+          $related_result->{__valiant_allow_destroy} = 1 if $allow_destroy;
           $related_result->set_from_params_recursively(%$params);
-
         } else {
           debug 3, "Updating with record from params (no matching PK)";
 
@@ -773,7 +893,9 @@ sub set_single_related_from_params {
             debug 3, 'update_only true';
             $related_result = $self->related_resultset($related)->single;
             unless($related_result) {
-              $related_result = $self->find_or_new_related($related, $params); # TODO should find from any unique keys only
+              #$related_result = $self->find_or_new_related($related, $params); # TODO should find from any unique keys on
+              # If one doesn't already exist that means 'make a new one'
+              $related_result = $self->new_related($related, $params);
             }
           } else {
             debug 3, "update_only false for rel $related on @{[ $self]}";
@@ -808,6 +930,7 @@ sub set_single_related_from_params {
             # my ($key) = keys(%$rev_data);
             #$related_result->set_from_related($key, $self) if $key; # Don't have this for might_hav
           }
+          $related_result->{__valiant_allow_destroy} = 1 if $allow_destroy;
           $related_result->set_from_params_recursively(%$params);
         }
       } else {
@@ -817,8 +940,25 @@ sub set_single_related_from_params {
         # there's nothing in the DB to find.
         #$related_result = $self->find_or_new_related($related, $params);
         #$related_result->set_from_params_recursively(%$params);
-
         my %local_params = %$params;
+
+        my %cond = %{$self->result_source->relationship_info($related)->{cond}};
+        my @pks = $self->result_source->primary_columns;
+
+        my $flag = 0;
+        foreach my $fk_proto (keys %cond) {
+          my $pk_proto = $cond{$fk_proto};
+          my ($pk) = ($pk_proto=~m/^self\.(.+)$/); 
+          my ($fk) = ($fk_proto=~m/^foreign\.(.+)$/);
+          next unless grep { $_ eq $pk } @pks; 
+          next unless exists $local_params{$fk};
+          $flag++ if defined($local_params{$fk});
+        }
+
+        if($flag == scalar(@pks)) {
+          DBIx::Class::Valiant::Util::Exception::BadParameters->throw(related=>$related, me=>$self);
+        }
+
         my %pk = map { $_ => delete $local_params{$_} }
           grep { exists $local_params{$_} }
           $self->related_resultset($related)->result_source->primary_columns;
@@ -829,6 +969,7 @@ sub set_single_related_from_params {
           debug 3, "setting with exact record matching pk %pk";
           $related_result = $self->result_source->related_source($related)->resultset->find(\%pk);
           #   $self->set_from_related($related, $related_result);
+          $related_result->{__valiant_allow_destroy} = 1 if $allow_destroy;
           $related_result->set_from_params_recursively(%$params);
         } else {
           debug 3, "No PK for $related, lets see if there's other ways to find it";
@@ -855,6 +996,7 @@ sub set_single_related_from_params {
             $related_result = $self->new_related($related, $params);
           }
           $self->set_from_related($related, $related_result) unless $self->in_storage;
+          $related_result->{__valiant_allow_destroy} = 1 if $allow_destroy;
           $related_result->set_from_params_recursively(%$params);
         }
       }
